# Copyright (C) 2005, 2006 by Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""Functionality for controlling a Patch Queue Manager (pqm).
"""

import os

from bzrlib.commands import Command, register_command
from bzrlib.option import Option
import bzrlib.errors as errors

from bzrlib.bzrdir import BzrDir


class cmd_pqm_submit(Command):
    """Submit the parent tree to the pqm.

    This acts like:
        $ echo "star-merge $PARENT $TARGET" 
            | gpg --cl 
            | mail pqm@somewhere -s "merge text"

    But it pays attention to who the local committer is 
    (using their e-mail address), and uses the local
    gpg signing configuration. (As well as target pqm
    settings, etc.)

    The reason we use 'parent' instead of the local branch
    is that most likely the local branch is not a public
    branch. And the branch must be available to the pqm.
    
    This can be configured at the branch level using ~/.bazaar/locations.conf.
    Here is an example:
        [/home/emurphy/repo]
        pqm_email = PQM <pqm@example.com>
        pqm_user_email = User Name <user@example.com>
        pqm_branch = http://code.example.com/code/project/devel
        public_repository = http://code.example.com/code/emurphy/project
        [/home/emurphy/repo/branch]
        # public_repository is used when public_branch is not set.
        # default behavior is to append the trailing path
        # Default value based on public_repository would be
        # http://code.example.com/code/emurphy/project/branch
        # override with
        public_branch = http://alternate.host.example.com/other/public/branch
                
        smtp_server = host:port
        smtp_username =
        smtp_password =
        
    If you don't specify the smtp server, the message will be sent via localhost.
    """

    takes_args = ['location?']
    takes_options = [
        Option('message',
               help='Message to use on merge to pqm.  '
                    'Currently must be a single line because of pqm limits.',
               type=str),
        Option('dry-run', help='Print request instead of sending.'),
        Option('public-location', type=str,
               help='Use this url as the public location to the pqm.'),
        ]

    def run(self, location=None, message=None, public_location=None, dry_run=False):
        from pqm_submit import submit

        if location is None:
            location = '.'
        # If we are simply specifying '.' then
        # we want to open the local branch.
        # all other uses require an exact path
        if location == '.':
            bzrdir = BzrDir.open_containing('.')[0]
        else:
            bzrdir = BzrDir.open(location)

        b = bzrdir.open_branch()
        submit(b, message=message, dry_run=dry_run, public_location=public_location)


register_command(cmd_pqm_submit)


def test_suite():
    from bzrlib.tests import TestSuite, TestLoader
    import test_pqm_submit

    loader = TestLoader()
    return loader.loadTestsFromModule(test_pqm_submit)
